@@ -1,27 +1,36 @@
-from filters import GenericFilterConverter
+from . import filters
 from ..base import BaseInterface
-from ..filters import Filters
+
+
+def _include_filters(obj):
+    for key in filters.__all__:
+        if not hasattr(obj, key):
+            setattr(obj, key, getattr(filters, key))
 
 
 class GenericInterface(BaseInterface):
 
+    filter_converter_class = filters.GenericFilterConverter
+
     def __init__(self, obj, session=None):
+        _include_filters(self)
         self.session = session
         super(GenericInterface, self).__init__(obj)
 
-
-    def get_filters(self, search_columns=[]):
-        return Filters(GenericFilterConverter, search_columns, self)
-
-
-    def query(self, filters=None, order_column='', order_direction='',
-              page=None, page_size=None):
+    def query(
+        self,
+        filters=None,
+        order_column="",
+        order_direction="",
+        page=None,
+        page_size=None,
+    ):
 
         query = self.session.query(self.obj)
         if filters:
             query = filters.apply_all(query)
-        if order_column != '':
-            query = query.order_by(order_column + ' ' + order_direction)
+        if order_column != "":
+            query = query.order_by(order_column + " " + order_direction)
         if page:
             query = query.offset(page * page_size)
         if page_size:
@@ -43,13 +52,18 @@ class GenericInterface(BaseInterface):
     def is_pk(self, col_name):
         return self.obj.properties[col_name].primary_key
 
+    def is_pk_composite(self):
+        return False
+
     def get_columns_list(self):
         return self.obj.columns
 
     def get_search_columns_list(self):
         return self.obj.columns
 
-    def get_order_columns_list(self):
+    def get_order_columns_list(self, list_columns=None):
+        if list_columns:
+            return list_columns
         return self.obj.columns
 
     def get_keys(self, lst):
@@ -64,5 +78,6 @@ class GenericInterface(BaseInterface):
             if self.is_pk(col_name):
                 return col_name
 
-    def get(self, id):
+    def get(self, id, filters=None):
+        # TODO: need to implement filters!
         return self.session.get(id)
